@@ -236,7 +236,7 @@ class XEP_0384(BasePlugin):
                                      "no data directory specified.")
 
         self._device_id = _load_device_id(self.data_dir)
-        asyncio.ensure_future(self.session_start_omemo())
+        asyncio.create_task(self.session_start_omemo())
 
         self.xmpp.add_event_handler('session_start', self.session_start)
         self.xmpp['xep_0060'].map_node_event(OMEMO_DEVICES_NS, 'omemo_device_list')
@@ -245,7 +245,7 @@ class XEP_0384(BasePlugin):
         # If this plugin is loaded after 'session_start' has fired, we still
         # need to publish bundles
         if self.xmpp.is_connected and not self._initial_publish_done:
-            asyncio.ensure_future(self._initial_publish())
+            asyncio.create_task(self._initial_publish())
 
     def plugin_end(self):
         if not self.backend_loaded:
@@ -290,8 +290,8 @@ class XEP_0384(BasePlugin):
         if self.backend_loaded:
             self.xmpp['xep_0163'].add_interest(OMEMO_DEVICES_NS)
             await asyncio.wait([
-                self._set_device_list(),
-                self._publish_bundle(),
+                asyncio.create_task(self._set_device_list()),
+                asyncio.create_task(self._publish_bundle()),
             ])
             self._initial_publish_done = True
 
@@ -299,7 +299,7 @@ class XEP_0384(BasePlugin):
         return self._device_id
 
     async def my_fingerprint(self) -> str:
-        bundle = await self._omemo().public_bundle.serialize(self.omemo_backend)
+        bundle = self._omemo().public_bundle.serialize(self.omemo_backend)
         return fp_from_ik(bundle['ik'])
 
     def _set_node_config(
@@ -415,9 +415,9 @@ class XEP_0384(BasePlugin):
             else:
                 log.debug('Not publishing.')
 
-    async def _fetch_bundle(self, jid: JID, device_id: int) -> None:
+    async def fetch_bundle(self, jid: JID, device_id: int) -> None:
         """
-            Fetch bundles for specified jid / device_id pair.
+            Fetch bundle for specified jid / device_id pair.
         """
         log.debug('Fetching bundle for JID: %r, device: %r', jid, device_id)
         node = f'{OMEMO_BUNDLES_NS}:{device_id}'
@@ -435,7 +435,23 @@ class XEP_0384(BasePlugin):
         else:
             log.debug('Encryption: Bundle %r not found!', device_id)
 
-    async def _fetch_devices(self, jid: JID) -> None:
+    async def fetch_bundles(self, jid: JID) -> None:
+        """
+            Fetch bundles of active devices for specified JID.
+
+            This is a helper function to allow the user to request a store
+            update. Failed bundles are not retried.
+        """
+        # Ignore failures
+        await asyncio.gather(
+            *map(
+                lambda did: self.fetch_bundle(jid, did),
+                await self.get_active_devices(jid)
+            ),
+            return_exceptions=True,
+        )
+
+    async def fetch_devices(self, jid: JID) -> None:
         """
             Manually query PEP OMEMO_DEVICES_NS nodes
         """
@@ -453,7 +469,7 @@ class XEP_0384(BasePlugin):
 
     def _receive_device_list(self, msg: Message) -> None:
         """Handler for received PEP OMEMO_DEVICES_NS payloads"""
-        asyncio.ensure_future(
+        asyncio.create_task(
             self._read_device_list(msg['from'], msg['pubsub_event']['items']),
         )
 
@@ -473,7 +489,7 @@ class XEP_0384(BasePlugin):
 
         return None
 
-    async def _set_device_list(self, device_ids: Optional[Set[int]] = None) -> None:
+    async def _set_device_list(self, device_ids: Optional[Iterable[int]] = None) -> None:
         own_jid = self.xmpp.boundjid
 
         try:
@@ -538,19 +554,19 @@ class XEP_0384(BasePlugin):
                     own_jid.bare, OMEMO_DEVICES_NS, payload=payload,
                 )
 
-    async def get_devices(self, jid: JID) -> Iterable[str]:
+    async def get_devices(self, jid: JID) -> Iterable[int]:
         """
             Get all devices for a JID.
         """
         devices = await self._omemo().getDevices(jid.bare)
-        return set(devices.get('active', []) + devices.get('inactive', []))
+        return map(int, set(devices.get('active', []) + devices.get('inactive', [])))
 
-    async def get_active_devices(self, jid: JID) -> Iterable[str]:
+    async def get_active_devices(self, jid: JID) -> Iterable[int]:
         """
             Return active device ids. Always contains our own device id.
         """
         devices = await self._omemo().getDevices(jid.bare)
-        return set(devices.get('active', []))
+        return map(int, set(devices.get('active', [])))
 
     async def _should_heartbeat(self, jid: JID, device_id: int, prekey: bool) -> bool:
         """
@@ -739,7 +755,7 @@ class XEP_0384(BasePlugin):
                 raise UntrustedException(exn.bare_jid, exn.device, exn.ik)
             raise
         finally:
-            asyncio.ensure_future(self._publish_bundle())
+            asyncio.create_task(self._publish_bundle())
 
         if self.auto_heartbeat:
             log.debug('Checking if heartbeat is required. auto_hearbeat enabled.')
@@ -829,10 +845,10 @@ class XEP_0384(BasePlugin):
             for exn in errors:
                 if isinstance(exn, omemo.exceptions.NoDevicesException):
                     log.debug('Encryption: Missing device list for JID: %r', exn.bare_jid)
-                    await self._fetch_devices(JID(exn.bare_jid))
+                    await self.fetch_devices(JID(exn.bare_jid))
                 elif isinstance(exn, omemo.exceptions.MissingBundleException):
                     log.debug('Encryption: Missing bundle for JID: %r, device: %r', exn.bare_jid, exn.device)
-                    await self._fetch_bundle(JID(exn.bare_jid), exn.device)
+                    await self.fetch_bundle(JID(exn.bare_jid), exn.device)
                 elif isinstance(exn, omemo.exceptions.TrustException):
                     # On TrustException, there are two possibilities.
                     # Either trust has not been explicitely set yet, and is
